@@ -144,6 +144,15 @@ namespace FSpot {
 			}
 		}
 		
+		private static Gdk.Rectangle Expand (Gdk.Rectangle src, int width)
+		{
+			src.X -= width;
+			src.Y -= width;
+			src.Width += width * 2;
+			src.Height += width * 2;
+			return src;
+		}
+
 		private void UpdateButtons () {
 			left.Sensitive = (scroll_offset < 0);
 			right.Sensitive = (box_counts.Length * BoxWidth > background.Width - scroll_offset);
@@ -632,10 +641,8 @@ namespace FSpot {
 			{
 				Rectangle box = InnerBounds ();
 
-				box.X -= border;
-				box.Y -= border;
-				box.Width += 2 * border;
-				box.Height += 2 * border + handle_height;
+				box = Expand (box, border);
+				box.Height += handle_height;
 				
 				return box;
 			}
@@ -646,27 +653,24 @@ namespace FSpot {
 				Rectangle bounds = Bounds ();
 				
 				if (bounds.Intersect (area, out area)) {
-					
-					
 					int i = 0;
+
 					Rectangle box = inner;
 					box.Width -= 1;
 					box.Height -= 1;
 					while (i < border) {
-						box.X -= 1;
-						box.Y -= 1;
-						box.Width += 2;
-						box.Height += 2;
-					
+						box = Expand (box, 1);
+						
+						selector.Style.BackgroundGC (State).ClipRectangle = area;
 						selector.GdkWindow.DrawRectangle (selector.Style.BackgroundGC (State), 
 										  false, box);
 						i++;
 					}
-				
+
 					Style.PaintHandle (selector.Style, selector.GdkWindow, State, ShadowType.In, 
 							    area, selector, "glass", bounds.X, inner.Y + inner.Height + border, 
 							    bounds.Width, handle_height, Orientation.Horizontal);
-
+					
 					Style.PaintShadow (selector.Style, selector.GdkWindow, State, ShadowType.Out, 
 							   area, selector, "glass", bounds.X, bounds.Y, bounds.Width, bounds.Height);
 
@@ -743,10 +747,10 @@ namespace FSpot {
 						    selector, "IconView:Selector", bottom_points, true);
 #else
 				Style.PaintBox (selector.Style, selector.GdkWindow, State, ShadowType.Out, area,
-						selector, null, top.X, top.Y, top.Width, top.Height);
+						selector, "limit", top.X, top.Y, top.Width, top.Height);
 
 				Style.PaintBox (selector.Style, selector.GdkWindow, State, ShadowType.Out, area,
-						selector, null, bottom.X, bottom.Y, bottom.Width, bottom.Height);
+						selector, "limit", bottom.X, bottom.Y, bottom.Width, bottom.Height);
 #endif
 			}
 
@@ -920,14 +924,16 @@ namespace FSpot {
 			min_limit = new Limit (this, Limit.LimitType.Min);
 			max_limit = new Limit (this, Limit.LimitType.Max);
 
-			left = new Gtk.Button (new Gtk.Image (Gtk.Stock.GoBack, Gtk.IconSize.Button));
+			left = new Gtk.Button ();
+			left.Add (new Gtk.Image (Gtk.Stock.GoBack, Gtk.IconSize.Button));
 			left.Relief = Gtk.ReliefStyle.None;
 			//left.Clicked += HandleScrollLeft;
 			left.Pressed += HandleLeftPressed;
 			left.ButtonReleaseEvent += HandleScrollReleaseEvent;
 			left_delay = new Delay (50, new GLib.IdleHandler (HandleScrollLeft));
 
-			right = new Gtk.Button (new Gtk.Image (Gtk.Stock.GoForward, Gtk.IconSize.Button));
+			right = new Gtk.Button ();
+			right.Add (new Gtk.Image (Gtk.Stock.GoForward, Gtk.IconSize.Button));
 			right.Relief = Gtk.ReliefStyle.None;
 			right.Pressed += HandleRightPressed;
 			right.ButtonReleaseEvent += HandleScrollReleaseEvent;
