@@ -21,7 +21,7 @@ public class IconView : Gtk.Layout {
 	// Public properties.
 
 	/* Width of the thumbnails. */
-	private int thumbnail_width = 128;
+	protected int thumbnail_width = 128;
 	public int ThumbnailWidth {
 		get {
 			return thumbnail_width;
@@ -35,7 +35,7 @@ public class IconView : Gtk.Layout {
 		}
 	}
 
-	private double thumbnail_ratio = 4.0 / 3.0;
+	protected double thumbnail_ratio = 4.0 / 3.0;
 	public double ThumbnailRatio {
 		get {
 			return thumbnail_ratio;
@@ -77,30 +77,30 @@ public class IconView : Gtk.Layout {
 	}
 
 	// Size of the frame around the thumbnail.
-	private const int CELL_BORDER_WIDTH = 10;
+	protected const int CELL_BORDER_WIDTH = 10;
 
 	// Border around the scrolled area.
-	private const int BORDER_SIZE = 6;
+	protected const int BORDER_SIZE = 6;
 
 	// Thickness of the outline used to indicate selected items. 
 	private const int SELECTION_THICKNESS = 5;
 
 	// Size of the tag icon in the view.
-	private const int TAG_ICON_SIZE = 16;
+	protected const int TAG_ICON_SIZE = 16;
 
 	// Horizontal spacing between the tag icons
-	private const int TAG_ICON_HSPACING = 2;
+	protected const int TAG_ICON_HSPACING = 2;
 
 	// Vertical spacing between the thumbnail and the row of tag icons.
-	private const int TAG_ICON_VSPACING = 6;
+	protected const int TAG_ICON_VSPACING = 3;
 
 	// The loader.
 	private PixbufLoader pixbuf_loader;
 
 	// Various other layout values.
-	private int cells_per_row;
-	private int cell_width;
-	private int cell_height;
+	protected int cells_per_row;
+	protected int cell_width;
+	protected int cell_height;
 
 	// The first pixel line that is currently on the screen (i.e. in the current
 	// scroll region).  Used to compute the area that went offscreen in the "changed"
@@ -174,9 +174,6 @@ public class IconView : Gtk.Layout {
 		this.collection = collection;
 	}
 	
-
-	protected IconView (IntPtr raw) : base (raw) {}
-
 	//
 	// IPhotoSelection
 	//
@@ -376,13 +373,13 @@ public class IconView : Gtk.Layout {
 	
 	// FIXME I can't find a c# wrapper for the C PANGO_PIXELS () macro
 	// So this Function is for that.
-	static int PangoPixels (int val)
+	protected static int PangoPixels (int val)
 	{
 		return val >= 0 ? (val + 1024 / 2) / 1024 :
 			(val - 1024 / 2) / 1024;
 	}
 	
-	private void UpdateLayout ()
+	protected virtual void UpdateLayout ()
 	{
 		int available_width = Allocation.Width - 2 * BORDER_SIZE;
 
@@ -416,7 +413,7 @@ public class IconView : Gtk.Layout {
 		Vadjustment.Change ();
 	}
 
-	Gdk.Rectangle Expand (Gdk.Rectangle src, int width)
+	static Gdk.Rectangle Expand (Gdk.Rectangle src, int width)
 	{
 		src.X -= width;
 		src.Y -= width;
@@ -491,7 +488,9 @@ public class IconView : Gtk.Layout {
 
 		Gdk.Rectangle region = Gdk.Rectangle.Zero;
 		Gdk.Rectangle image_bounds = Expand (bounds, - CELL_BORDER_WIDTH);
-		if (image_bounds.Intersect (area, out image_bounds) && thumbnail != null) {
+		int expansion = ThrobExpansion (thumbnail_num, selected);
+
+		if (Expand (image_bounds, expansion + 1).Intersect (area, out image_bounds) && thumbnail != null) {
 			
 			PixbufUtils.Fit (thumbnail, ThumbnailWidth, ThumbnailHeight, 
 					 true, out region.Width, out region.Height);
@@ -499,7 +498,6 @@ public class IconView : Gtk.Layout {
 			region.X = (int) (bounds.X + (bounds.Width - region.Width) / 2);
 			region.Y = (int) bounds.Y + ThumbnailHeight - region.Height + CELL_BORDER_WIDTH;
 			
-			int expansion = ThrobExpansion (thumbnail_num, selected);
 
 			if (region.Width != thumbnail.Width || region.Height != thumbnail.Height)
 				pixbuf_loader.Request (thumbnail_path, thumbnail_num);
@@ -586,7 +584,7 @@ public class IconView : Gtk.Layout {
 			Tag [] tags = photo.Tags;
 			Gdk.Rectangle tag_bounds;
 
-			tag_bounds.X = bounds.X + (bounds.Width - tags.Length * TAG_ICON_SIZE) / 2;
+			tag_bounds.X = bounds.X + (bounds.Width  + TAG_ICON_VSPACING - tags.Length * (TAG_ICON_SIZE + TAG_ICON_VSPACING)) / 2;
 			tag_bounds.Y = bounds.Y + bounds.Height - CELL_BORDER_WIDTH - TAG_ICON_SIZE;
 			tag_bounds.Width = TAG_ICON_SIZE;
 			tag_bounds.Height = TAG_ICON_SIZE;
@@ -665,6 +663,12 @@ public class IconView : Gtk.Layout {
 
 	private void GetCellPosition (int cell_num, out int x, out int y)
 	{
+		if (cells_per_row == 0) {
+			x = 0;
+			y = 0;
+			return;
+		}
+			
 		int row = cell_num / cells_per_row;
 		int col = cell_num % cells_per_row;
 
@@ -771,6 +775,9 @@ public class IconView : Gtk.Layout {
 
 	public void ScrollTo (int cell_num)
 	{
+		if (!IsRealized)
+			return;
+
 		Adjustment adjustment = Vadjustment;
 		int x;
 		int y;
@@ -973,30 +980,37 @@ public class IconView : Gtk.Layout {
 
 		focus_old = FocusCell;
 		switch (args.Event.Key) {
-			case Gdk.Key.Down:
-				FocusCell += cells_per_row;
+		case Gdk.Key.Down:
+			FocusCell += cells_per_row;
+			break;
+		case Gdk.Key.Left:
+			FocusCell--;
+			break;
+		case Gdk.Key.Right:
+			FocusCell++;
+			break;
+		case Gdk.Key.Up:
+			FocusCell -= cells_per_row;
+			break;
+		case Gdk.Key.Home:
+			FocusCell = 0;
+			break;
+		case Gdk.Key.End:
+			FocusCell = collection.Items.Length - 1; 
+			break;
+		case Gdk.Key.space:
+			ToggleCell (FocusCell);
+			break;
+		case Gdk.Key.Return:
+			if (DoubleClicked == null)
 				break;
-			case Gdk.Key.Left:
-				FocusCell--;
+			if (FocusCell < 0 || FocusCell > collection.Items.Length - 1)
 				break;
-			case Gdk.Key.Right:
-				FocusCell++;
-				break;
-			case Gdk.Key.Up:
-				FocusCell -= cells_per_row;
-				break;
-			case Gdk.Key.Home:
-				FocusCell = 0;
-				break;
-			case Gdk.Key.End:
-				FocusCell = collection.Items.Length - 1; 
-				break;
-			case Gdk.Key.space:
-				ToggleCell (FocusCell);
-				break;
-			default:	
-				args.RetVal = false;
-				return;		
+			DoubleClicked (this, FocusCell);
+			break;
+		default:	
+			args.RetVal = false;
+			return;		
 		}
 		
 		if (FocusCell < 0 || FocusCell > collection.Items.Length - 1) {
