@@ -10,15 +10,12 @@ using System.Collections;
 using System.Runtime.InteropServices;
 using System.Text.RegularExpressions;
 
+using LibGPhoto2;
+
 public class MainWindow {
         public static MainWindow Toplevel = null;
 
 	Db db;
-	public Db Database {
-		get {
-			return db;
-		}
-	}
 
 	TagSelectionWidget tag_selection_widget;
 	[Glade.Widget] Gtk.Window main_window;
@@ -63,6 +60,9 @@ public class MainWindow {
 	[Glade.Widget] MenuItem find_tag;
 
 	PhotoVersionMenu versions_submenu;
+
+	Gtk.ToggleButton browse_button;
+	Gtk.ToggleButton view_button;
 	
 	InfoBox info_box;
 	FSpot.InfoDisplay info_display;
@@ -71,9 +71,12 @@ public class MainWindow {
 	FSpot.FullScreenView fsview;
 	FSpot.PhotoQuery query;
 	FSpot.GroupSelector group_selector;
-
+	
 	FSpot.Delay slide_delay;
 	
+	string last_import_path;
+	ModeType view_mode;
+
 	// Drag and Drop
 	enum TargetType {
 		UriList,
@@ -103,14 +106,138 @@ public class MainWindow {
 	};
 
 	const int PHOTO_IDX_NONE = -1;
-	
-	struct YearCount {
-		int Year;
-		int Count;
+
+	//
+	// Constructor
+	//
+	public MainWindow (Db db)
+	{
+		this.db = db;
+
+		Mono.Posix.Catalog.Init ("f-spot", FSpot.Defines.LOCALE_DIR);
+		Glade.XML gui = Glade.XML.FromAssembly ("f-spot.glade", "main_window", null);
+		gui.Autoconnect (this);
+
+		slide_delay = new FSpot.Delay (new GLib.IdleHandler (SlideShow));
+
+		Gtk.Toolbar toolbar = new Gtk.Toolbar ();
+		toolbar_vbox.PackStart (toolbar);
+		GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-270", new System.EventHandler (HandleRotate270Command));
+		GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-90", new System.EventHandler (HandleRotate90Command));
+		toolbar.AppendSpace ();
+		browse_button = GtkUtil.MakeToolbarToggleButton (toolbar, "f-spot-browse", 
+								 new System.EventHandler (HandleToggleViewBrowse)) as ToggleButton;
+		view_button = GtkUtil.MakeToolbarToggleButton (toolbar, "f-spot-edit-image", 
+							       new System.EventHandler (HandleToggleViewPhoto)) as ToggleButton;
+		toolbar.AppendSpace ();
+		GtkUtil.MakeToolbarButton (toolbar, "f-spot-fullscreen", new System.EventHandler (HandleViewFullscreen));
+		GtkUtil.MakeToolbarButton (toolbar, "f-spot-slideshow", new System.EventHandler (HandleViewSlideShow));
+
+		tag_selection_widget = new TagSelectionWidget (db.Tags);
+		tag_selection_scrolled.Add (tag_selection_widget);
+		
+		tag_selection_widget.Selection.Changed += HandleTagSelectionChanged;
+		tag_selection_widget.SelectionChanged += OnTagSelectionChanged;
+		tag_selection_widget.DragDataGet += HandleTagSelectionDragDataGet;
+		tag_selection_widget.DragDrop += HandleTagSelectionDragDrop;
+		Gtk.Drag.SourceSet (tag_selection_widget, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask,
+				    tag_target_table, DragAction.Copy | DragAction.Move);
+
+		tag_selection_widget.DragDataReceived += HandleTagSelectionDragDataReceived;
+		tag_selection_widget.DragMotion += HandleTagSelectionDragMotion;
+		Gtk.Drag.DestSet (tag_selection_widget, DestDefaults.All, tag_dest_target_table, 
+				  DragAction.Copy | DragAction.Move ); 
+
+		tag_selection_widget.ButtonPressEvent += HandleTagSelectionButtonPressEvent;
+
+		info_box = new InfoBox ();
+		info_box.VersionIdChanged += HandleInfoBoxVersionIdChange;
+		left_vbox.PackStart (info_box, false, true, 0);
+
+		query = new FSpot.PhotoQuery (db.Photos);
+		query.ItemChanged += HandleQueryItemChanged;
+
+#if SHOW_CALENDAR
+		FSpot.SimpleCalendar cal = new FSpot.SimpleCalendar (query);
+		cal.DaySelected += HandleCalendarDaySelected;
+		left_vbox.PackStart (cal, false, true, 0);
+#endif
+
+		group_selector = new FSpot.GroupSelector ();
+		FSpot.GroupAdaptor adaptor = new FSpot.TimeAdaptor (query);
+
+		group_selector.Adaptor  = adaptor;
+		group_selector.ShowAll ();
+
+		view_vbox.PackStart (group_selector, false, false, 0);
+		view_vbox.ReorderChild (group_selector, 0);
+
+		icon_view = new QueryView (query);
+		icon_view_scrolled.Add (icon_view);
+		icon_view.SelectionChanged += HandleSelectionChanged;
+		icon_view.DoubleClicked += HandleDoubleClicked;
+		icon_view.GrabFocus ();
+
+		new FSpot.PreviewPopup (icon_view);
+
+		Gtk.Drag.SourceSet (icon_view, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask,
+				    icon_source_target_table, DragAction.Copy | DragAction.Move);
+		
+		icon_view.DragBegin += HandleIconViewDragBegin;
+		icon_view.DragDataGet += HandleIconViewDragDataGet;
+
+		TagMenu menu = new TagMenu (attach_tag, db.Tags);
+		menu.TagSelected += HandleAttachTagMenuSelected;
+
+		menu = new TagMenu (find_tag, db.Tags);
+		menu.TagSelected += HandleFindTagMenuSelected;
+
+		PhotoTagMenu pmenu = new PhotoTagMenu ();
+		pmenu.TagSelected += HandleRemoveTagMenuSelected;
+		remove_tag.Submenu = pmenu;
+		
+		Gtk.Drag.DestSet (icon_view, DestDefaults.All, icon_dest_target_table, 
+				  DragAction.Copy | DragAction.Move); 
+
+		//		icon_view.DragLeave += new DragLeaveHandler (HandleIconViewDragLeave);
+		icon_view.DragMotion += HandleIconViewDragMotion;
+		icon_view.DragDrop += HandleIconViewDragDrop;
+		icon_view.DragDataReceived += HandleIconViewDragDataReceived;
+
+		photo_view = new PhotoView (query, db.Photos);
+		photo_box.Add (photo_view);
+		photo_view.PhotoChanged += HandlePhotoViewPhotoChanged;
+		photo_view.ButtonPressEvent += HandlePhotoViewButtonPressEvent;
+		photo_view.KeyPressEvent += HandlePhotoViewKeyPressEvent;
+		photo_view.UpdateStarted += HandlePhotoViewUpdateStarted;
+		photo_view.UpdateFinished += HandlePhotoViewUpdateFinished;
+
+		Gtk.Drag.DestSet (photo_view, DestDefaults.All, tag_target_table, 
+				  DragAction.Copy | DragAction.Move); 
+
+		photo_view.DragMotion += HandlePhotoViewDragMotion;
+		photo_view.DragDrop += HandlePhotoViewDragDrop;
+		photo_view.DragDataReceived += HandlePhotoViewDragDataReceived;
+
+		view_notebook.SwitchPage += HandleViewNotebookSwitchPage;
+		adaptor.GlassSet += HandleAdaptorGlassSet;
+
+		UpdateMenus ();
+		main_window.ShowAll ();
+		main_window.Destroyed += HandleCloseCommand;
+
+		if (Toplevel == null)
+			Toplevel = this;
+
+		UpdateToolbar ();
+
+		if (db.Empty)
+			HandleImportCommand (null, null);
 	}
 
 	// Index into the PhotoQuery.  If -1, no photo is selected or multiple photos are selected.
-	private int ActiveIndex () {
+	private int ActiveIndex () 
+	{
 		if (view_mode == ModeType.IconView && icon_view.CurrentIdx != -1)
 			return icon_view.CurrentIdx;
 
@@ -136,127 +263,16 @@ public class MainWindow {
 		}
 	}
 
+	public Db Database {
+		get {
+			return db;
+		}
+	}
 
-	//
-	// Constructor
-	//
-	public MainWindow (Db db)
-	{
-		this.db = db;
-
-		Mono.Posix.Catalog.Init ("f-spot", FSpot.Defines.LOCALE_DIR);
-		Glade.XML gui = Glade.XML.FromAssembly ("f-spot.glade", "main_window", null);
-		gui.Autoconnect (this);
-
-		slide_delay = new FSpot.Delay (new GLib.IdleHandler (SlideShow));
-
-		Gtk.Toolbar toolbar = new Gtk.Toolbar ();
-		toolbar_vbox.PackStart (toolbar);
-		GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-270", new System.EventHandler (HandleRotate270Command));
-		GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-90", new System.EventHandler (HandleRotate90Command));
-		toolbar.AppendSpace ();
-		GtkUtil.MakeToolbarButton (toolbar, "f-spot-browse", new System.EventHandler (HandleViewBrowse));
-		GtkUtil.MakeToolbarButton (toolbar, "f-spot-edit-image", new System.EventHandler (HandleViewPhoto));
-		toolbar.AppendSpace ();
-		GtkUtil.MakeToolbarButton (toolbar, "f-spot-fullscreen", new System.EventHandler (HandleViewFullscreen));
-		GtkUtil.MakeToolbarButton (toolbar, "f-spot-slideshow", new System.EventHandler (HandleViewSlideShow));
-
-		tag_selection_widget = new TagSelectionWidget (db.Tags);
-		tag_selection_scrolled.Add (tag_selection_widget);
-		
-		tag_selection_widget.Selection.Changed += new EventHandler (HandleTagSelectionChanged);
-		tag_selection_widget.SelectionChanged += new TagSelectionWidget.SelectionChangedHandler (OnTagSelectionChanged);
-		tag_selection_widget.DragDataGet += new DragDataGetHandler (HandleTagSelectionDragDataGet);
-		Gtk.Drag.SourceSet (tag_selection_widget, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask,
-				    tag_target_table, DragAction.Copy | DragAction.Move);
-
-		tag_selection_widget.DragDataReceived += new DragDataReceivedHandler (HandleTagSelectionDragDataReceived);
-		tag_selection_widget.DragMotion += new DragMotionHandler (HandleTagSelectionDragMotion);
-		Gtk.Drag.DestSet (tag_selection_widget, DestDefaults.All, tag_dest_target_table, 
-				  DragAction.Copy); 
-
-		tag_selection_widget.ButtonPressEvent += new ButtonPressEventHandler (HandleTagSelectionButtonPressEvent);
-
-		info_box = new InfoBox ();
-		info_box.VersionIdChanged += new InfoBox.VersionIdChangedHandler (HandleInfoBoxVersionIdChange);
-		left_vbox.PackStart (info_box, false, true, 0);
-
-		query = new FSpot.PhotoQuery (db.Photos);
-		query.ItemChanged += HandleQueryItemChanged;
-
-#if SHOW_CALENDAR
-		FSpot.SimpleCalendar cal = new FSpot.SimpleCalendar (query);
-		cal.DaySelected += HandleCalendarDaySelected;
-		left_vbox.PackStart (cal, false, true, 0);
-#endif
-
-		group_selector = new FSpot.GroupSelector ();
-		FSpot.GroupAdaptor adaptor = new FSpot.TimeAdaptor (query);
-		//FSpot.GroupAdaptor adaptor = new FSpot.DirectoryAdaptor (query);		
-		group_selector.Mode = FSpot.GroupSelector.RangeType.All;
-
-		group_selector.Adaptor  = adaptor;
-		group_selector.ShowAll ();
-
-		view_vbox.PackStart (group_selector, false, false, 0);
-		view_vbox.ReorderChild (group_selector, 0);
-
-		icon_view = new QueryView (query);
-		icon_view_scrolled.Add (icon_view);
-		icon_view.SelectionChanged += new IconView.SelectionChangedHandler (HandleSelectionChanged);
-		icon_view.DoubleClicked += new IconView.DoubleClickedHandler (HandleDoubleClicked);
-		icon_view.GrabFocus ();
-
-		new FSpot.PreviewPopup (icon_view);
-
-		Gtk.Drag.SourceSet (icon_view, Gdk.ModifierType.Button1Mask | Gdk.ModifierType.Button3Mask,
-				    icon_source_target_table, DragAction.Copy | DragAction.Move);
-		
-		icon_view.DragBegin += new DragBeginHandler (HandleIconViewDragBegin);
-		icon_view.DragDataGet += new DragDataGetHandler (HandleIconViewDragDataGet);
-
-		TagMenu menu = new TagMenu (attach_tag, db.Tags);
-		menu.TagSelected += HandleAttachTagMenuSelected;
-
-		menu = new TagMenu (find_tag, db.Tags);
-		menu.TagSelected += HandleFindTagMenuSelected;
-
-		PhotoTagMenu pmenu = new PhotoTagMenu ();
-		pmenu.TagSelected += HandleRemoveTagMenuSelected;
-		remove_tag.Submenu = pmenu;
-		
-		Gtk.Drag.DestSet (icon_view, DestDefaults.All, icon_dest_target_table, 
-				  DragAction.Copy | DragAction.Move); 
-
-		//		icon_view.DragLeave += new DragLeaveHandler (HandleIconViewDragLeave);
-		icon_view.DragMotion += new DragMotionHandler (HandleIconViewDragMotion);
-		icon_view.DragDrop += new DragDropHandler (HandleIconViewDragDrop);
-		icon_view.DragDataReceived += new DragDataReceivedHandler (HandleIconViewDragDataReceived);
-
-		photo_view = new PhotoView (query, db.Photos);
-		photo_box.Add (photo_view);
-		photo_view.PhotoChanged += HandlePhotoViewPhotoChanged;
-		photo_view.ButtonPressEvent += HandlePhotoViewButtonPressEvent;
-		photo_view.KeyPressEvent += HandlePhotoViewKeyPressEvent;
-		photo_view.UpdateStarted += HandlePhotoViewUpdateStarted;
-		photo_view.UpdateFinished += HandlePhotoViewUpdateFinished;
-
-		Gtk.Drag.DestSet (photo_view, DestDefaults.All, tag_target_table, 
-				  DragAction.Copy | DragAction.Move); 
-
-		photo_view.DragMotion += new DragMotionHandler (HandlePhotoViewDragMotion);
-		photo_view.DragDrop += new DragDropHandler (HandlePhotoViewDragDrop);
-		photo_view.DragDataReceived += new DragDataReceivedHandler (HandlePhotoViewDragDataReceived);
-
-		view_notebook.SwitchPage += new SwitchPageHandler (HandleViewNotebookSwitchPage);
-		adaptor.GlassSet += HandleAdaptorGlassSet;
-
-		UpdateMenus ();
-		main_window.ShowAll ();
-		main_window.Destroyed += HandleCloseCommand;
-
-		if (Toplevel == null)
-			Toplevel = this;
+	public ModeType ViewMode {
+		get {
+			return view_mode;
+		}
 	}
 
 	// Switching mode.
@@ -265,14 +281,6 @@ public class MainWindow {
 		PhotoView
 	};
 
-	ModeType view_mode;
-
-	public ModeType ViewMode {
-		get {
-			return view_mode;
-		}
-	}
-
 	public void SetViewMode (ModeType value)
 	{
 		view_mode = value;
@@ -290,8 +298,27 @@ public class MainWindow {
 			Present (icon_view.FocusCell);
 			break;
 		}
+		UpdateToolbar ();
 	}
 	
+	void UpdateToolbar ()
+	{
+		if (browse_button != null) {
+			bool state = view_mode == ModeType.IconView;
+			
+			if (browse_button.Active != state)
+				browse_button.Active = state;
+		}
+
+		if (view_button != null) {
+			bool state = view_mode == ModeType.PhotoView;
+			
+			if (view_button.Active != state)
+				view_button.Active = state;
+		}
+	}
+		
+
 	void HandleViewNotebookSwitchPage (object sender, SwitchPageArgs args)
 	{
 		switch (view_notebook.CurrentPage) {
@@ -306,7 +333,7 @@ public class MainWindow {
 
 
 	public int [] SelectedIds () {
-		int [] ids;
+		int [] ids = new int [0];
 
 		if (fsview != null)
 			ids = new int [] { fsview.View.CurrentPhoto };
@@ -317,7 +344,8 @@ public class MainWindow {
 				break;
 			default:
 			case ModeType.PhotoView:
-				ids = new int [] { photo_view.CurrentPhoto };
+				if (photo_view.View.CurrentPhotoValid ())
+					ids = new int [] { photo_view.CurrentPhoto };
 				break;
 			}
 		}
@@ -416,6 +444,25 @@ public class MainWindow {
 		}
 	}
 
+	void HandleTagSelectionDragDataGet (object sender, DragDataGetArgs args)
+	{		
+		UriList list = new UriList (SelectedPhotos ());
+
+		switch (args.Info) {
+		case (uint) TargetType.TagList:
+			Byte [] data = Encoding.UTF8.GetBytes (list.ToString ());
+			Atom [] targets = args.Context.Targets;
+		
+			args.SelectionData.Set (targets[0], 8, data, data.Length);
+			break;
+		} 
+	}
+
+	void HandleTagSelectionDragDrop (object sender, DragDropArgs args)
+	{
+		args.RetVal = true;
+	}
+
 	public void HandleTagSelectionDragMotion (object o, DragMotionArgs args)
 	{
 		TreePath path;
@@ -460,15 +507,6 @@ public class MainWindow {
 		}
 	}
 
-	void HandleTagSelectionDragDataGet (object sender, DragDataGetArgs args)
-	{		
-		UriList list = new UriList (SelectedPhotos ());
-		Byte [] data = Encoding.UTF8.GetBytes (list.ToString ());
-		Atom [] targets = args.Context.Targets;
-		
-		args.SelectionData.Set (targets[0], 8, data, data.Length);
-	}
-
 #if SHOW_CALENDAR
 	void HandleCalendarDaySelected (object sender, System.EventArgs args)
 	{
@@ -552,20 +590,19 @@ public class MainWindow {
 
 	void HandleIconViewDragDataGet (object sender, DragDataGetArgs args)
 	{		
-		UriList list = new UriList (SelectedPhotos ());
-
 		switch (args.Info) {
-		case (uint) TargetType.TagList:
+		case (uint) TargetType.UriList:
+		case (uint) TargetType.PhotoList:
+			UriList list = new UriList (SelectedPhotos ());
 			Byte [] data = Encoding.UTF8.GetBytes (list.ToString ());
 			Atom [] targets = args.Context.Targets;
-		
 			args.SelectionData.Set (targets[0], 8, data, data.Length);
 			break;
 		case (uint) TargetType.RootWindow:
-			System.Console.WriteLine ("Got background drop");
 			HandleSetAsBackgroundCommand (null, null);
-			break;
-		} 
+                        break;
+		}
+		       
 	}
 
 	void HandleIconViewDragDrop (object sender, DragDropArgs args)
@@ -596,7 +633,7 @@ public class MainWindow {
 	void HandleIconViewDragDataReceived (object sender, DragDataReceivedArgs args)
 	{
 	 	Widget source = Gtk.Drag.GetSourceWidget (args.Context);     
-
+		
 		switch (args.Info) {
 		case (uint)TargetType.TagList:
 			//
@@ -607,12 +644,12 @@ public class MainWindow {
 							     args.Y + (int) icon_view.Vadjustment.Value);
 
 			//Console.WriteLine ("Drop cell = {0} ({1},{2})", item, args.X, args.Y);
-
-			if (icon_view.CellIsSelected (item))
-				AttachTags (tag_selection_widget.TagHighlight (), SelectedIds());
-			else 
-				AttachTags (tag_selection_widget.TagHighlight (), new int [] {item});
-
+			if (item >= 0) {
+				if (icon_view.CellIsSelected (item))
+					AttachTags (tag_selection_widget.TagHighlight (), SelectedIds());
+				else 
+					AttachTags (tag_selection_widget.TagHighlight (), new int [] {item});
+			}
 			break;
 		case (uint)TargetType.UriList:
 
@@ -765,12 +802,62 @@ public class MainWindow {
 
 	void HandleImportCommand (object sender, EventArgs e)
 	{
+		db.Sync = false;
 		ImportCommand command = new ImportCommand (main_window);
-		if (command.ImportFromFile (db.Photos) > 0) {
+		if (command.ImportFromFile (db.Photos, this.last_import_path) > 0) {
+			this.last_import_path = command.ImportPath;
 			UpdateQuery ();
 		}
+		db.Sync = true;		
 	}
 
+	void HandleImportFromCameraCommand (object sender, EventArgs e)
+	{
+		GPhotoCamera cam = new GPhotoCamera();
+		
+		try {
+			int num_cameras = cam.DetectCameras();
+			int selected_cam;
+
+			if (num_cameras < 1) {
+				HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
+									    MessageType.Warning, ButtonsType.Ok, 
+									    Mono.Posix.Catalog.GetString ("No cameras detected."),
+									    Mono.Posix.Catalog.GetString ("F-Spot was unable to find any cameras attached to this system.  Double check that the camera is connected and has power")); 
+
+				md.Run ();
+				md.Destroy ();
+				return;
+			} else if (num_cameras == 1) {
+				selected_cam = 0;
+			} else {
+				FSpot.CameraSelectionDialog camselect = new FSpot.CameraSelectionDialog (cam.CameraList);
+				selected_cam = camselect.Run ();
+			}
+			
+			if (selected_cam >= 0) {
+				cam.SelectCamera (selected_cam);	
+				cam.InitializeCamera ();
+			
+				FSpot.CameraFileSelectionDialog selector = new FSpot.CameraFileSelectionDialog (cam, db);
+				selector.Run ();
+
+				UpdateQuery ();
+			}
+		}
+		catch (GPhotoException ge) {
+			HigMessageDialog md = new HigMessageDialog (main_window, DialogFlags.DestroyWithParent, 
+								    MessageType.Error, ButtonsType.Ok, 
+								    Mono.Posix.Catalog.GetString ("Error connecting to camera"),
+								    String.Format (Mono.Posix.Catalog.GetString ("Received error \"{0}\" while connecting to camera"), 
+										   ge.Message));
+			md.Run ();
+			md.Destroy ();
+		} finally {
+			cam.ReleaseGPhotoResources ();
+		}
+	}
+	
 	unsafe void HandlePrintCommand (object sender, EventArgs e)
 	{
 		new FSpot.PrintDialog (SelectedPhotos ());
@@ -815,6 +902,11 @@ public class MainWindow {
 		new FSpot.VfsExport (new FSpot.PhotoArray (SelectedPhotos ()));
 	}
 
+	void HandleExportToOriginal (object sender, EventArgs args)
+	{
+		new FSpot.OriginalExport (new FSpot.PhotoArray (SelectedPhotos ()));
+	}
+
 	void HandleViewDirectory (object sender, EventArgs args)
 	{
 		Gtk.Window win = new Gtk.Window ("Directory View");
@@ -870,9 +962,10 @@ public class MainWindow {
 			"Tambet Ingo",
 			"MOREAU Vincent",
 			"Lee Willis",
+			"Alessandro Gervaso",
 			"Peter Johanson",
 			"Grahm Orr",
-			"Cheslack-Postava",
+			"Ewen Cheslack-Postava",
 			"Patanjali Somayaji",
 			"Matt Jones",
 			"Martin Willemoes Hansen",
@@ -896,7 +989,7 @@ public class MainWindow {
 		group_selector.Adaptor.GlassSet -= HandleAdaptorGlassSet;
 		FSpot.GroupAdaptor adaptor = new FSpot.TimeAdaptor (query);
 		group_selector.Adaptor = adaptor;
-		group_selector.Mode = FSpot.GroupSelector.RangeType.All;
+		group_selector.Mode = FSpot.GroupSelector.RangeType.Min;
 		adaptor.GlassSet += HandleAdaptorGlassSet;
 	}
 
@@ -962,13 +1055,14 @@ public class MainWindow {
 	public void HandleCreateNewTagCommand (object sender, EventArgs args)
 	{
 		TagCommands.Create command = new TagCommands.Create (db.Tags, main_window);
-		command.Execute (TagCommands.TagType.Tag);
+		command.Execute (TagCommands.TagType.Tag, tag_selection_widget.TagHighlight ());
+		
 	}
 
 	public void HandleCreateNewCategoryCommand (object sender, EventArgs args)
 	{
 		TagCommands.Create command = new TagCommands.Create (db.Tags, main_window);
-		command.Execute (TagCommands.TagType.Category);
+		command.Execute (TagCommands.TagType.Category, tag_selection_widget.TagHighlight ());
 	}
 
 	public void HandleAttachTagCommand (object obj, EventArgs args)
@@ -1049,6 +1143,7 @@ public class MainWindow {
 				Gdk.Pixbuf final = PixbufUtils.UnsharpMask (orig, radius_spin.Value, amount_spin.Value, threshold_spin.Value);
 			
 				Photo photo = query.Photos [id];
+				Exif.ExifData exif_data = new Exif.ExifData (photo.DefaultVersionPath);
 
 				uint version = photo.DefaultVersionId;
 				if (version == Photo.OriginalVersionId) {
@@ -1058,7 +1153,7 @@ public class MainWindow {
 				try {
 					string version_path = photo.GetVersionPath (version);
 					
-					final.Savev (version_path, "jpeg", null, null);
+					PixbufUtils.SaveJpeg (final, version_path, 95, exif_data);
 					FSpot.ThumbnailGenerator.Create (version_path).Dispose ();
 					photo.DefaultVersionId = version;
 					query.Commit (id);
@@ -1179,6 +1274,27 @@ public class MainWindow {
 		args.RetVal = true;
 	}
 
+	void HandleToggleViewBrowse (object sender, EventArgs args)
+	{
+	        ToggleButton toggle = sender as ToggleButton;
+		if (toggle != null) {
+			if (toggle.Active)
+				SetViewMode (ModeType.IconView);
+		} else
+			SetViewMode (ModeType.IconView);
+	}
+
+	void HandleToggleViewPhoto (object sender, EventArgs args)
+	{
+	        ToggleButton toggle = sender as ToggleButton;
+		
+		if (toggle != null) {
+			if (toggle.Active)
+				SetViewMode (ModeType.PhotoView);
+		} else
+			SetViewMode (ModeType.IconView);
+	}
+
 	void HandleViewBrowse (object sender, EventArgs args)
 	{
 		SetViewMode (ModeType.IconView);
@@ -1272,28 +1388,49 @@ public class MainWindow {
 			break;
 		}
 	}
+	
+	public void HandleDeleteCommand (object sender, EventArgs args)
+	{
+   		Photo[] photos = SelectedPhotos();
+   		string header = Mono.Posix.Catalog.GetPluralString ("Delete the selected photo permanently?", 
+								 "Delete the {0} selected photos permanently?", 
+								 photos.Length);
+		header = String.Format (header, photos.Length);
+		string msg = Mono.Posix.Catalog.GetPluralString ("This deletes all versions of the selected photo from your drive.", 
+								 "This deletes all versions of the selected photos from your drive.", 
+								 photos.Length);
+		string ok_caption = Mono.Posix.Catalog.GetPluralString ("_Delete photo", "_Delete photos", photos.Length);
+		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(main_window, DialogFlags.DestroyWithParent, MessageType.Warning, header, msg, ok_caption)) {                              
+			foreach (Photo photo in photos) {
+				foreach (uint id in photo.VersionIds) {
+					Console.WriteLine (" path == {0}", photo.GetVersionPath (id)); 
+					photo.DeleteVersion (id, true);
+				}
 
-        public void HandleDeleteCommand (object sender, EventArgs args)
-        {
-		foreach (Photo photo in SelectedPhotos ()) {
-			foreach (uint id in photo.VersionIds) {
-				Console.WriteLine (" path == {0}", photo.GetVersionPath (id)); 
-				photo.DeleteVersion (id, true);
+				db.Photos.Remove (photo);
 			}
 
-			db.Photos.Remove (photo);
+			UpdateQuery ();
 		}
-
-		UpdateQuery ();
 	}
 
 	public void HandleRemoveCommand (object sender, EventArgs args)
 	{
-		foreach (Photo photo in SelectedPhotos ()) {
-			db.Photos.Remove (photo);
-		}
+   		Photo[] photos = SelectedPhotos();
+   		string header = Mono.Posix.Catalog.GetPluralString ("Remove the selected photo from F-Spot?",
+								 "Remove the {0} selected photos from F-Spot?", 
+								 photos.Length);
 
-		UpdateQuery ();
+		header = String.Format (header, photos.Length);
+		string msg = Mono.Posix.Catalog.GetString("If you remove photos from the F-Spot catalog all tag information will be lost. The photos remain on your computer and can be imported into F-Spot again.");
+		string ok_caption = Mono.Posix.Catalog.GetString("_Remove from Catalog");
+		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(main_window, DialogFlags.DestroyWithParent, MessageType.Warning, header, msg, ok_caption)) {                              
+			foreach (Photo photo in photos) {
+				db.Photos.Remove (photo);
+			}
+
+			UpdateQuery ();
+		}
 	}
 
 	void HandleSelectAllCommand (object sender, EventArgs args)
@@ -1309,9 +1446,18 @@ public class MainWindow {
 	public void HandleDeleteSelectedTagCommand (object sender, EventArgs args)
 	{
 		Tag [] tags = this.tag_selection_widget.TagHighlight ();
-		
-		db.Photos.Remove (tags);
-		icon_view.QueueDraw ();
+ 		string header = Mono.Posix.Catalog.GetPluralString ("Delete the selected tag?",
+								 "Delete the {0} selected tags?", 
+								 tags.Length);
+
+		header = String.Format (header, tags.Length);
+		string msg = Mono.Posix.Catalog.GetString("If you delete a tag, all associations with photos are lost.");
+		string ok_caption = Mono.Posix.Catalog.GetPluralString ("_Delete tag", "_Delete tags", tags.Length);
+		 
+ 		if (ResponseType.Ok == HigMessageDialog.RunHigConfirmation(main_window, DialogFlags.DestroyWithParent, MessageType.Warning, header, msg, ok_caption)) {                              
+ 			db.Photos.Remove (tags);
+			icon_view.QueueDraw ();
+ 		}
 	}
 
 	void HandleUpdateThumbnailCommand (object sende, EventArgs args)
@@ -1389,7 +1535,7 @@ public class MainWindow {
 	{
 		CurrentPhoto.DefaultVersionId = version_id;
 		int active = ActiveIndex ();
-
+		
 		query.Commit (active);
 	}
 
