@@ -81,11 +81,13 @@ class PixbufUtils {
 
 			double scale = Math.Min (max_width / (double)args.Width,
 						 max_height / (double)args.Height);
+
 			
 			int scale_width = (int)(scale * args.Width);
 			int scale_height = (int)(scale * args.Height);
 
-			SetSize (scale_width, scale_height);
+			if (scale < 1.0)
+				SetSize (scale_width, scale_height);
 		}
 
 		public Pixbuf LoadFromFile (string path)
@@ -118,16 +120,25 @@ class PixbufUtils {
 	}
 
 	public static Pixbuf ScaleToMaxSize (Pixbuf pixbuf, int width, int height)
+	{
+		return ScaleToMaxSize (pixbuf, width, height, true);
+	}	
+
+	public static Pixbuf ScaleToMaxSize (Pixbuf pixbuf, int width, int height, bool upscale)
 	{
 		double scale = Math.Min  (width / (double)pixbuf.Width, height / (double)pixbuf.Height);
 		int scale_width = (int)(scale * pixbuf.Width);
 		int scale_height = (int)(scale * pixbuf.Height);
 
-		Gdk.Pixbuf result = pixbuf.ScaleSimple (scale_width, scale_height, Gdk.InterpType.Bilinear);
+		Gdk.Pixbuf result;
+		if (upscale || (scale < 1.0))
+			result = pixbuf.ScaleSimple (scale_width, scale_height, Gdk.InterpType.Bilinear);
+		else
+			result = pixbuf.Copy ();
 
 		CopyThumbnailOptions (pixbuf, result);
 
-		return pixbuf.ScaleSimple (scale_width, scale_height, Gdk.InterpType.Bilinear);
+		return result;
 	}
 		
 	static public Pixbuf LoadAtMaxSize (string path, int max_width, int max_height)
@@ -282,6 +293,26 @@ class PixbufUtils {
 		return scaled;
 	}
 
+	public static string Resize (string orig_path, int size, bool copy_meta)
+	{
+		string version_path = System.IO.Path.GetTempFileName ();
+		Resize (orig_path, version_path, size, copy_meta);
+		return version_path;
+	}
+
+	public static void Resize (string orig_path, string dest_path, int size, bool copy_meta)
+	{
+		Exif.ExifData exif_data;
+		if (copy_meta)
+			exif_data = new Exif.ExifData (orig_path);
+		else 
+			exif_data = new Exif.ExifData ();
+
+		Gdk.Pixbuf image = PixbufUtils.LoadAtMaxSize (orig_path, size, size);
+		PixbufUtils.SaveJpeg (image, dest_path, 95, exif_data);
+	}
+	
+
 	[StructLayout(LayoutKind.Sequential)]
 	public unsafe struct FPixbufJpegMarker {
 		public int type;
@@ -346,6 +377,20 @@ class PixbufUtils {
 		return ret;
 	}	
 	
+	[DllImport ("libfspot")]
+	static extern void f_pixbuf_remove_redeye (IntPtr src);
+	
+	public static Gdk.Pixbuf RemoveRedeye (Gdk.Pixbuf src, Gdk.Rectangle area)
+	{
+		Gdk.Pixbuf copy = src.Copy ();
+		Gdk.Pixbuf selection = new Gdk.Pixbuf (copy, area.X, area.Y, area.Width, area.Height);
+
+		f_pixbuf_remove_redeye (selection.Handle);
+		selection.Dispose ();
+
+		return copy;
+	}
+
 	public static unsafe Pixbuf ColorAdjust (Pixbuf src, double brightness, double contrast,
 					  double hue, double saturation, int src_color, int dest_color)
 	{
